#include <fitsio.h>
#include "yse.h"
#include "yse_fits.h"
#include "yse_field.h"

int yse_fits_load(yse_field_t *field, const char *filename)
{
	int status = 0;
	long width, height;
	double FWHM = 0., FWHMH, FWHMHS, FWHMV, FWHMVS;
	fitsfile *file;
	char comment[FLEN_COMMENT];
	long naxis;

	fits_open_file(&file, filename, READONLY, &status);
	if (status != 0)
	{
		fits_report_error(stderr, status);
		return -1;
	}
	
	fits_read_key_lng(file, "NAXIS", &naxis, comment, &status);

	if (naxis != 2)
	{
		fits_close_file(file, &status);
		return -1;
	}

	fits_read_key_lng(file, "NAXIS1", &width, comment, &status);
	fits_read_key_lng(file, "NAXIS2", &height, comment, &status);
	//fits_read_key_dbl(file, "FWHMH", &FWHMH, comment, &status);
	//fits_read_key_dbl(file, "FWHMHS", &FWHMHS, comment, &status);
	//fits_read_key_dbl(file, "FWHMV", &FWHMV, comment, &status);
	//fits_read_key_dbl(file, "FWHMVS", &FWHMVS, comment, &status);
	
	//FWHM = (FWHMH + FWHMV) / 2.;
	FWHM = DEFAULTFWHM;
	
	if (status != 0)
	{
		fits_close_file(file, &status);
		return -1;
	}

	if (yse_field_init(field, FWHM, width, height) < 0)
	{
		fits_close_file(file, &status);
		return -1;
	}
	
	fits_read_img(file, TFLOAT, 1L, field->width * field->height,
			NULL, field->data_buffer, NULL, &status);

	fits_close_file(file, &status);
	if (status != 0)
		fits_report_error(stderr, status);

	return 0;
}
