/*-------------------------------------------------------------
				find trail on astro photo

		we assume that the trail is almost a line.
		
			Kim, Dae-Won && Kim, Soo-Young
	    Department of Astronomy, Yonsei University
-------------------------------------------------------------*/
/*---------------------------------------YSTAR PROJECT-------*/

#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <sys/time.h>
#include <unistd.h>
#include <stdlib.h>
#include <string.h>

#include <fitsio.h>

#include "yse.h"
#include "yse_fits.h"
#include "yse_field.h"

#include "yse_line_list.h"
#include "yse_star_list.h"
#include "yse_line.h"

#include "yse_background.h"

/***************************************************/
//finding information for calcurating flatness and peak and etc
void find_distance_info(yse_line_list *head, float centerX, float centerY, //input value
	float *minmaxDistance, int *minmaxValue); //return value
	
void delete_searched_area(yse_field_t *field, yse_line_list *head, int num_of_pix_line); //input value

//void sort(float *array, int size);
/***************************************************/

void find_distance_info(yse_line_list *head, float centerX, float centerY, 
	float *minmaxDistance, int *minmaxValue)
{
	float distance = 0.0;
	int minX = 0, maxX = 0;
	int minY = 0, maxY = 0;
	
	yse_line_list *line = NULL;
	
	minmaxDistance[0] = 0.0; //minDistance
	minmaxDistance[1] = 0.0; //maxDistance
	
	minmaxValue[0] = 0; //minX
	minmaxValue[1] = 0; //maxX
	minmaxValue[2] = 0; //minY
	minmaxValue[3] = 0; //maxY
	
	line = head;
	//find distance from center
	while(line != NULL)
	{
		distance = sqrt((line->x - centerX) * (line->x - centerX) + (line->y - centerY) * (line->y - centerY));
		
		if(minmaxDistance[0] == 0.0)
		{
			minmaxDistance[0] = distance;
			minmaxDistance[1] = distance;
			minX = line->x;
			maxX = line->x;
			minY = line->y;
			maxY = line->y;
		}
		else
		{
			if(distance != 0)
			{
				if(distance > minmaxDistance[1])
					minmaxDistance[1] = distance;
				else if(distance < minmaxDistance[0])
					minmaxDistance[0] = distance;
			}
		
			if(minmaxValue[0] > line->x)
				minmaxValue[0] = line->x;
			if(minmaxValue[1] < line->x)
				minmaxValue[1] = line->x;
			
			if(minmaxValue[2] > line->y)
				minmaxValue[2] = line->y;
			if(minmaxValue[3] < line->y)
				minmaxValue[3] = line->y;
		}
		line = line->next;
	}
}

void delete_searched_area(yse_field_t *field, yse_line_list *head, int num_pix_of_line)
{
	int k;
	int scanY;
	int minX, maxX;
	//printf("start of delete\n");
	yse_line_list *line = NULL;
	
	if(num_pix_of_line > 1)
	{
		while(head != NULL)
		{
			scanY = head->y;
			minX = head->x; maxX = head->x;
			
			line = head;
			while(line != NULL)
			{
				if(head->y == line->y)
				{
					if(line->x < minX)
						minX = line->x;
					else if(line->x > maxX)
						maxX = line->x;
				}
				line = line->next;
			}
			
			for(k = minX; k <= maxX; k++)
			{
				field->data[scanY][k] = 0;
			}
			head = head->next;
		}
	}
	//printf("end of delete\n");
}

/*
void sort(float *array, int size)
{
	int i, j;
	float dummy;
	
	for(i = 0; i < size; i++)
		for(j = i + 1; j < size; j++)
		{
			if(array[j - 1] < array[j])
			{
				dummy = array[j];
				array[j] = array[j - 1];
				array[j - 1] = dummy;
			}
		}
}
*/

int yse_main(const char *filename, float threshold, float min_length_streak)
{
	int i, j, k;
	float lower_limit = 0.;
	float rms[2];
	
	int num_longer_than_MLS = 0; // MLS = min_length_streak
	int num_corr[1];
	
	int lower_line_size = DEFAULTFWHM * 6.;
	int upper_line_size = 2000;
	
	yse_field_t _field, *field = &_field;
		
	//for finding border
	int startX = -1, startY = -1;
	int currX = -1, currY = -1;
	int scanX = -1, scanY = -1;
	float centerX = 0.0, centerY = 0.0;
	int direction = 0;
	int found = 0;
	int num_pix_of_line = 1;
	int total_num_of_line = 0;
	
	//store line data
	yse_line_list *head = NULL, *line = NULL;
	
	//for determining line or not
	float minmaxDistance[2];
	float flatness = 0.0;
	
	//for min, max Value
	int minmaxValue[4];
	int minX = 0, maxX = 0;
	int minY = 0, maxY = 0;
	float slope = 0.0;

	yse_star_list *star, *dummy;
	char EXEC[256];
	
	int exit_process = 0;
	
	printf("#------------[ %s ] IS PROCESSING------------\n", filename);
	
	num_corr[0] = 0;
	
	yse_fits_load(field, filename);
	
	//sprintf(EXEC, "%s %s", SEX_EXEC, filename);
	//system(EXEC);
	
	star = malloc(sizeof(yse_star_list));
	star->next = NULL;
	dummy = malloc(sizeof(yse_star_list));
	dummy->next = NULL;
	
	//yse_star_list_get(dummy);
	star = dummy->next;
	
	yse_background_estimate(field, rms);
	if(min_length_streak != 0.)
		//lower_limit = rms[0] + rms[1] * 2.;
		lower_limit = rms[0] + rms[1] * 3.;
	else
		lower_limit = rms[0] + rms[1] * 5.;
		//lower_limit = rms[0] + rms[1] * 3.;
		
	if(lower_limit > 1000. || lower_limit < -1000.)
		lower_limit = threshold;
	
	if(field->FWHM != 0.)
	{
		if(min_length_streak != 0.)
			lower_line_size = (field->FWHM * 8);
			//lower_line_size = (field->FWHM * 1.);
		else
			//lower_line_size = (field->FWHM * 1.);
			lower_line_size = (field->FWHM * 3.);
	}		
	
	printf("#FWHM %f lower_limit %f lower_line_size %d WIDTH %d HEIGHT %d\n", field->FWHM, lower_limit, lower_line_size, field->width, field->height);
	
	for(i = field->height - 1; i >= 0; i--)
	{
		for(j = 0; j < field->width; j++)
		{		
			/***************************************************/
			//border tracing start
			if(field->data[i][j] > lower_limit)
			{
				startX = j; startY = i;
				
				line = malloc(sizeof(yse_line_list));
				head = malloc(sizeof(yse_line_list));
				head->next = NULL;
				line->next = NULL;
				
				line->x = startX; line->y = startY;
				//this is boundary line
				head = line;
				
				centerX = startX; centerY = startY;
				//printf("start at %d %d\n", startX, startY);
				while(!(currX == startX && currY == startY))
				{
					if(currX == -1)
					{
						currX = startX; currY = startY;
					}
					
					for(k = 0; k < 8; k++)
					{
						if(direction < 0)
							direction += 8;
						direction = direction % 8;
						
						switch(direction)
						{
							case 0:
								scanX = currX + 1; scanY = currY;
								break;
							case 1:
								scanX = currX + 1; scanY = currY + 1;
								break;
							case 2:
								scanX = currX; scanY = currY + 1;
								break;
							case 3:
								scanX = currX - 1; scanY = currY + 1;
								break;
							case 4:
								scanX = currX - 1; scanY = currY;
								break;
							case 5:
								scanX = currX - 1; scanY = currY - 1;
								break;
							case 6:
								scanX = currX; scanY = currY - 1;
								break;
							case 7:
								scanX = currX + 1; scanY = currY - 1;
								break;
						}
						if((scanX > field->width - 1 || scanX < 0) || (scanY > field->height - 1 || scanY < 0))
						{
							direction -= 1;
							continue;
						}
						//printf("scan at %d %d %f %d \n", scanX, scanY, field->data[scanY][scanX], direction);
						if(field->data[scanY][scanX] > lower_limit)
						{
							if(!(currX == startX && currY == startY))
							{
								line->next = malloc(sizeof(yse_line_list));
								line = line->next;
								//printf("%d %d:", currX, currY);
								line->x = currX; line->y = currY;
								line->next = NULL;
								
								centerX += currX; centerY += currY;
								num_pix_of_line += 1;
							}
							
							currX = scanX; currY = scanY;
							
							direction += 2;
							found = 1;
							break;
						}
						
						direction -= 1;
					}
				}
				//border tracing end
				/***************************************************/
				if(num_pix_of_line < lower_line_size || num_pix_of_line > upper_line_size)
				{
					;
				}
				else
				{
					centerX = centerX / (float)num_pix_of_line;
					centerY = centerY / (float)num_pix_of_line;
					
					find_distance_info(head, centerX, centerY, minmaxDistance, minmaxValue);
					
					minX = minmaxValue[0];
					maxX = minmaxValue[1];
					minY = minmaxValue[2];
					maxY = minmaxValue[3];
					
					flatness = 1.0 - minmaxDistance[0] / minmaxDistance[1];
					//slope must be recalcurated from line of boundary's center
					slope = ((float)(maxY - minY) / (float)(maxX - minX));
					
					//yse_line_print(head);
					
					if(flatness > 0.75)
					{
						//printf("#check flatness...\n");
						total_num_of_line += yse_line_process(field, head, star, NULL, filename, rms, min_length_streak);
					}
				}
				
				//area filling
				delete_searched_area(field, head, num_pix_of_line);
				
				//setting all variable to initial value for preparing next.
				head = NULL;
				line = NULL;
				free(line);
				free(head);
				
				startX = -1; startY = -1;
				currX = -1; currY = -1;
				scanX = -1; scanY = -1;
				direction = 0;
				found = 0;
				num_pix_of_line = 1;
				minX = 0; maxX = 0;
				minY = 0; maxY = 0;
				centerX = 0.0; centerY = 0.0;
				slope = 0.0;
			}
		}
	}
	
	//too many streaks
	if(min_length_streak != 0.)
	{
		if(total_num_of_line > MAX_STREAK_NUM_BEF_COR)
			exit_process = 1;
	}
	else
	{
		if(total_num_of_line > MAX_STREAK_NUM)
			exit_process = 1;
	}
	
	if(exit_process)
	{
		printf("#Too many streaks are founded. This image may be not qualified.\n");
		sprintf(EXEC, "%s.out.txt", filename);
		unlink(EXEC);
		return -1;
	}
	
	if(total_num_of_line >= 1)
	{
		printf("#Find a correlation between streaks...\n");
		yse_find_correlation(filename, field, min_length_streak);
	}
	
	if(total_num_of_line >= 1)
	{
		printf("#Make result text for jpeg images...\n");
		num_longer_than_MLS = yse_make_text_for_jpeg(filename, field, min_length_streak, num_corr);
	}
	
	printf("#We founed total %d streaks and get finally %d streaks by self correlation.\n", total_num_of_line, num_corr[0]);
	
	if(num_corr[0] > MAX_STREAK_NUM)
	{
		printf("#Too many corrleated streaks are founded. This image may be not qualified.\n");
		sprintf(EXEC, "%s.out.txt", filename);
		unlink(EXEC);
		sprintf(EXEC, "%s.dat", filename);
		unlink(EXEC);
		return -1;
	}
	
	if(min_length_streak != 0.)
		if(num_longer_than_MLS == 0)
			printf("#There is no streaks longer than %5.2f.\n", min_length_streak);
		else
		{
			printf("#There is %d streaks longer than %5.2f.\n", num_longer_than_MLS, min_length_streak);
			//yse_make_jpeg(filename, field, rms);
		}
	
	//free all memory allocation variable;
	field = NULL;
	free(field);
	
	return 0;
}

int main(int argc, char **argv)
{
	char *filename;
	double threshold = 0.;
	float min_length_streak = 0.;
	
	if (argc < 2)
	{
		printf("./streak fits_filename spare_threshold [minimum length of streaks]\n");
		return -2;
	}

	filename = argv[1];
	if(argc >= 3)
		threshold = atof(argv[2]);
	if(argc >= 4)
		min_length_streak = atof(argv[3]);
	
	return yse_main(filename, threshold, min_length_streak);
}
